import Icon from "./Icon"
import Player from "./Player"
import { faSpinnerThird } from "@fortawesome/pro-solid-svg-icons"
import { FontAwesomeIcon } from "@fortawesome/react-fontawesome"
import { useGameProps } from "@hooks/useGameProps"
import useSocket from "@hooks/useSocket"
import { socket } from "@lib/socket"
import { useSession } from "next-auth/react"
import { useRouter } from "next/router"
import { Fragment, useEffect, useState } from "react"

function WithDots({ children }: { children: string }) {
  const [dots, setDots] = useState(3)

  useEffect(() => {
    const interval = setInterval(() => setDots((e) => (e % 3) + 1), 1000)
    return () => clearInterval(interval)
  }, [])
  return (
    <>
      {children + " "}
      {Array.from(Array(dots), () => ".").join("")}
      {Array.from(Array(3 - dots), (_, i) => (
        <Fragment key={i}>&nbsp;</Fragment>
      ))}
    </>
  )
}

function LobbyFrame({ openSettings }: { openSettings: () => void }) {
  const { payload, full, leave, reset } = useGameProps()
  const { isConnected } = useSocket()
  const router = useRouter()
  const { data: session } = useSession()

  useEffect(() => {
    if (payload?.game?.id || !isConnected) return
    socket.emit("update", full)
  }, [full, payload?.game?.id, isConnected])

  return (
    <div className="mx-32 flex flex-col self-stretch rounded-3xl bg-gray-400">
      <div className="flex items-center justify-between border-b-2 border-slate-900">
        <Icon src="speech_bubble.png">Chat</Icon>
        <h1 className="font-farro text-5xl font-medium">
          Game-PIN:{" "}
          {isConnected ? (
            <span className="underline">{payload?.gamePin ?? "----"}</span>
          ) : (
            <FontAwesomeIcon icon={faSpinnerThird} spin={true} />
          )}
        </h1>
        <Icon src="gear.png" onClick={openSettings}>
          Settings
        </Icon>
      </div>
      <div className="flex items-center justify-around">
        <Player
          src="player_blue.png"
          player={payload?.player1}
          userId={session?.user.id}
        />
        <p className="font-farro m-4 text-6xl font-semibold">VS</p>
        {payload?.player2 ? (
          <Player
            src="player_red.png"
            player={payload?.player2}
            userId={session?.user.id}
          />
        ) : (
          <p className="font-farro w-96 text-center text-4xl font-medium">
            <WithDots>
              {"Warte auf " + (isConnected ? "Spieler 2" : "Verbindung")}
            </WithDots>
          </p>
        )}
      </div>
      <div className="flex items-center justify-center border-t-2 border-slate-900">
        <button
          className="font-farro mx-32 my-4 rounded-xl border-b-4 border-red-400 bg-red-500 px-12 py-4 text-5xl font-medium duration-100 active:border-t-4 active:border-b-0"
          onClick={() => {
            leave(async () => {
              await router.push("/")
              reset()
            })
          }}
        >
          LEAVE
        </button>
        <button className="font-farro mx-32 my-4 rounded-xl border-b-4 border-orange-400 bg-warn px-12 py-4 text-5xl font-medium duration-100 active:border-t-4 active:border-b-0">
          START
        </button>
      </div>
    </div>
  )
}

export default LobbyFrame
