import { useGameProps } from "./useGameProps"
import { socket } from "@lib/socket"
import status from "http-status"
import { useRouter } from "next/router"
import { useEffect, useState } from "react"
import { toast } from "react-toastify"

/** This function should only be called once per page, otherwise there will be multiple socket connections and duplicate event listeners. */
function useSocket() {
  const [isConnected, setIsConnected] = useState(false)
  const { setPlayer, setSetting, full, hash: stateHash } = useGameProps()
  const router = useRouter()

  useEffect(() => {
    socket.connect()

    socket.on("connect", () => {
      setIsConnected(true)
      console.log("connected")

      const toastId = "connect_error"
      toast.dismiss(toastId)
    })

    socket.on("connect_error", (error) => {
      console.log("Connection error:", error.message)
      if (error.message === status["403"]) router.push("/")
      if (error.message !== "xhr poll error") return
      const toastId = "connect_error"
      const isActive = toast.isActive(toastId)
      console.log(toastId, isActive)
      if (isActive)
        toast.update(toastId, {
          autoClose: 5000,
        })
      else
        toast.warn("Es gibt Probleme mit der Echtzeitverbindung.", { toastId })
    })

    socket.on("gameSetting", (payload, hash) => {
      const newHash = setSetting(payload)
      if (!newHash || newHash === hash) return
      console.log("hash", hash, newHash)
      socket.emit("update", (body) => {
        console.log("update")
        full(body)
      })
    })

    socket.on("playerEvent", (payload, hash, type) => {
      let message: string
      switch (type) {
        case "disconnect":
          message = "Player is disconnected."
          break

        case "leave":
          message = "Player has left the lobby."
          break

        case "join":
          if (hash === stateHash || !stateHash) return
          message = "Player has joined the lobby."
          break

        default:
          message = "Not defined yet."
          break
      }
      toast.info(message, { toastId: message })
      console.log(payload)
      const newHash = setPlayer(payload)
      console.log(newHash, hash, !newHash, newHash === hash)
      if (!newHash || newHash === hash) return
      console.log("hash", hash, newHash)
      socket.emit("update", (body) => {
        console.log("update")
        full(body)
      })
    })

    socket.on("disconnect", () => {
      console.log("disconnect")
      setIsConnected(false)
    })

    return () => {
      socket.removeAllListeners()
      socket.disconnect()
    }
  }, [])

  // useEffect(() => {
  //   if (!isConnected) return
  //   let count = 0
  //   const interval = setInterval(() => {
  //     const start = Date.now()
  //     socket.volatile.emit("ping", ++count, (count) => {
  //       const duration = Date.now() - start
  //       console.log("ping", count, duration)
  //     })
  //   }, 5000)
  //   return () => clearInterval(interval)
  // }, [isConnected])

  return { isConnected }
}

export default useSocket
